﻿using System;
using System.IO;
using System.Runtime.InteropServices;

using WXImage = System.IntPtr;
using WXIDPhoto = System.IntPtr;

// C:\\work\\wx_human_seg\\release\\windows\\3.1.0\\idphoto_desktop\\bin\\x64\\wx_idphoto_desktop_sdk.dll
namespace test_idphoto_csharp
{
   
    public class WXIDPhotoSDK
    {
        [StructLayout(LayoutKind.Sequential)]
        public struct IDPhotoConfig
        {
            public int version;
            public int width;
            public int height;
            public float faceRatio;
            public float eyesTop;

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
            public float[] topEmpty;

            public WXImage bgImage;
            public ushort dpi;
            public IDPhotoConfig(int width, int height)
            {
                this.version = 0;
                this.width = width;
                this.height = height;
                this.faceRatio = 0.5f;
                this.eyesTop = 0.45f;
                this.topEmpty = new float[2];
                this.bgImage = WXImage.Zero;
                this.dpi = 300;

            }
            
        }

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXSDK_Version", CallingConvention = CallingConvention.StdCall)]
        public static extern IntPtr Version();

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXSDKInitWithLicenseFile", CallingConvention = CallingConvention.Cdecl)]
        public static extern int InitWithLicenseFile(string appKey, string licenseFilePath);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXImage_FromFile", CallingConvention = CallingConvention.Cdecl)]
        public static extern int ImageFromFile(string fileName, ref WXImage outImg);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXImage_FromColor", CallingConvention = CallingConvention.Cdecl)]
        public static extern int ImageFromColor(int width, int height, byte r, byte g, byte b, ref WXImage outImg);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXImage_Release", CallingConvention = CallingConvention.Cdecl)]
        public static extern int ImageRelease(WXImage image);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXImage_ToFile", CallingConvention = CallingConvention.Cdecl)]
        public static extern int ImageSave(WXImage image, string fileName);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXIDPhoto_Create", CallingConvention = CallingConvention.Cdecl)]
        public static extern int IDPhotoCreate(int deviceType, ref WXIDPhoto outIDPhoto);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXIDPhoto_MakeIDPhoto", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MakeIDPhoto(WXIDPhoto idPhotoObj, WXImage inImage, ref IDPhotoConfig config, IntPtr preResult, ref WXImage resultImg);

        [DllImport("wx_idphoto_desktop_sdk.dll", EntryPoint = "WXIDPhoto_Release", CallingConvention = CallingConvention.Cdecl)]
        public static extern int IDPhotoRelease(WXIDPhoto idPhotoObj);
    }

    class Program
    {
        static void TestMakeIDPhoto(string inputPath, string outputPath, WXIDPhoto idphotoObj)
        {
            WXImage inImage = WXImage.Zero;
            WXIDPhotoSDK.IDPhotoConfig config = new WXIDPhotoSDK.IDPhotoConfig(480, 640);
            WXImage resultImage = WXImage.Zero;
            int res = 0;
            try
            {
                res = WXIDPhotoSDK.ImageFromFile(inputPath, ref inImage);
                if (res != 0)
                {
                    Console.WriteLine("failed in ImageFromFile: " + res);
                    return;
                }

                res = WXIDPhotoSDK.ImageFromColor(config.width, config.height,
                                                  255, 0, 0, ref config.bgImage); // red color background
                if (res != 0)
                {
                    Console.WriteLine("failed in ImageFromColor: " + res);
                    return;
                }

                IntPtr preDetectResult = new IntPtr(0);
                res = WXIDPhotoSDK.MakeIDPhoto(idphotoObj, inImage, ref config, preDetectResult, ref resultImage); // 0 for cpu device mode
                if (res != 0)
                {
                    Console.WriteLine("failed in MakeIDPhoto: " + res);
                    return;
                }

                res = WXIDPhotoSDK.ImageSave(resultImage, outputPath);
                if (res != 0)
                {
                    Console.WriteLine("failed in ImageSave: " + res);
                    return;
                }

            }
            finally
            {

                if (inImage != WXImage.Zero)
                {
                    WXIDPhotoSDK.ImageRelease(inImage);
                }
                if (resultImage != WXImage.Zero)
                {
                    WXIDPhotoSDK.ImageRelease(resultImage);
                }
                if (config.bgImage != WXImage.Zero)
                {
                    WXIDPhotoSDK.ImageRelease(config.bgImage);
                }
                
            }
        }
        static void Main(string[] args)
        {
            string inputPath = "C:\\work\\wx_human_seg\\tests\\test1.jpg";
            string outputPath = "C:\\work\\wx_human_seg\\tests\\c_sharp_res.jpg";
            string licensePath = "C:\\work\\wx_human_seg\\tests_res\\idphoto_desktop.lic";

            WXIDPhoto idphotoObj = WXIDPhoto.Zero;

            try
            {
                IntPtr pRet = WXIDPhotoSDK.Version();
                string verion = Marshal.PtrToStringAnsi(pRet);

                Console.WriteLine("sdk version: " + verion);

                int res = WXIDPhotoSDK.InitWithLicenseFile(null, licensePath);
                if (res != 0)
                {
                    Console.WriteLine("failed in InitWithLicenseFile: " + res);
                    return;
                }

                // XXXX 非常重要的提示：
                // 创建WXIDPhoto操作对象，
                // 但WXIDPhoto对象最好不要每次制作一张证件照都创建一次，
                // 因为创建它存在一定的消耗，
                // 最好是创建一个全局的WXIDPhoto对象，
                // 每次制作证件照都使用这个全局的。
                res = WXIDPhotoSDK.IDPhotoCreate(0, ref idphotoObj); // 0 for cpu device mode
                if (res != 0)
                {
                    Console.WriteLine("failed in IDPhotoCreate: " + res);

                    return;
                }

                // 使用全局的WXIDPhoto对象制作证件照，并保存到文件
                TestMakeIDPhoto(inputPath, outputPath, idphotoObj);

                Console.WriteLine("Test finish");
            }
            finally
            {

                if (idphotoObj != WXIDPhoto.Zero)
                {
                    WXIDPhotoSDK.IDPhotoRelease(idphotoObj);
                }
            }
        }
    }
}
